using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;

class Program
{
	class Node
	{
		public override string ToString()
		{
			return index.ToString();
		}

		public int index;
		public bool inTree;
		public Arc treeArc;
		public NodePair pair;
		public Node mate;
		public List<Arc> outgoingArcs;
		public Queue<Arc> unfetchedOutgoingArcs;
	}

	class Arc
	{
		public override string ToString()
		{
			return string.Format("{0}->{1}", tail, head);
		}

		public Arc mate;
		public Node head;
		public Node tail;
	}

	class NodePair
	{
		public bool trimMark;
		public int dsuRank;
		public NodePair dsuParent;
		public Bud maxBud;
		public Bud ownerBud;
		public Node first;
		public Node second;
		public int enterTime;
		public int leaveTime;
		public LinkedListNode<NodePair> topLevelNode;
	}

	class Bud
	{
		public Arc baseArc;
		public Node baseNode;
		public readonly List<Arc> spanningArcs = new List<Arc>();
		public readonly List<Node> spanningNodes = new List<Node>();
		public Queue<Node> unfetchedSpanningNodes;
		public int enterTime;
		public int leaveTime;
	}

	readonly List<Node> nodes = new List<Node>();
	readonly List<Arc> arcs = new List<Arc>();
	readonly Queue<Node> treeNodes = new Queue<Node>();
	readonly LinkedList<NodePair> topLevelPairs = new LinkedList<NodePair>();

	int answer;

	NodePair GetDsuRoot(NodePair pair)
	{
		if (pair.dsuParent != pair)
			pair.dsuParent = GetDsuRoot(pair.dsuParent);
		return pair.dsuParent;
	}

	void DsuUnite(NodePair a, NodePair b)
	{
		a = GetDsuRoot(a);
		b = GetDsuRoot(b);
		if (a == b)
			return;
		if (a.dsuRank == b.dsuRank)
		{
			a.dsuRank++;
		}
		if (a.dsuRank < b.dsuRank)
			a.dsuParent = b;
		else
			b.dsuParent = a;
	}

	void DsuInit(NodePair pair)
	{
		pair.dsuParent = pair;
		pair.dsuRank = 0;
	}

	IEnumerable<Arc> FetchOutArcsInternal(Node node)
	{
		if (node.outgoingArcs != null)
		{
			if (node.unfetchedOutgoingArcs == null)
				node.unfetchedOutgoingArcs = new Queue<Arc>(node.outgoingArcs);
			while (node.unfetchedOutgoingArcs.Count > 0)
			{
				Arc arc = node.unfetchedOutgoingArcs.Dequeue();
				yield return arc;
			}
			yield break;
		}

		Bud bud = node.pair.ownerBud;
		Debug.Assert(node == bud.baseNode || node == bud.baseNode.mate);

		if (bud.unfetchedSpanningNodes == null)
			bud.unfetchedSpanningNodes = new Queue<Node>(bud.spanningNodes);

		while (bud.unfetchedSpanningNodes.Count > 0)
		{
			Node spanningNode = bud.unfetchedSpanningNodes.Peek();
			IEnumerable<Arc> outArcs = FetchOutArcsInternal(spanningNode);
			foreach (Arc arc in outArcs)
				yield return arc;

			Node spanningNodeMate = spanningNode.mate;
			outArcs = FetchOutArcsInternal(spanningNodeMate);
			foreach (Arc arc in outArcs)
				yield return arc;
			
			bud.unfetchedSpanningNodes.Dequeue();
		}
	}

	IEnumerable<Arc> FetchOutArcs(Node node)
	{
		Debug.Assert(node.pair.topLevelNode != null);
		if (node.outgoingArcs != null)
		{
			if (node.unfetchedOutgoingArcs == null)
				node.unfetchedOutgoingArcs = new Queue<Arc>(node.outgoingArcs);
			while (node.unfetchedOutgoingArcs.Count > 0)
			{
				Arc arc = node.unfetchedOutgoingArcs.Dequeue();
				yield return arc;
			}
			yield break;
		}

		Bud bud = GetDsuRoot(node.pair).maxBud;
		Debug.Assert(node == bud.baseNode || node == bud.baseNode.mate);
		if (node == bud.baseNode)
		{
			IEnumerable<Arc> outArcs = FetchOutArcsInternal(node);
			foreach (Arc arc in outArcs)
			{
				if ((GetDsuRoot(arc.head.pair).maxBud == null ||
				     GetDsuRoot(arc.head.pair) != GetDsuRoot(arc.tail.pair)) &&
				    arc != bud.baseArc.mate)
					yield return arc;
			}
		}
		else
			yield return bud.baseArc.mate;
	}

	void InitAugmentation()
	{
		treeNodes.Clear();
		topLevelPairs.Clear();
		foreach (Node node in nodes)
		{
			if (node == node.pair.first)
			{
				DsuInit(node.pair);
				node.pair.maxBud = null;
				node.pair.ownerBud = null;
				node.pair.topLevelNode = topLevelPairs.AddLast(node.pair);
			}
			node.treeArc = null;
			node.inTree = false;
			node.unfetchedOutgoingArcs = null;
		}
	}
	
	Node AntiLiftNode(Node node)
	{
		NodePair rootPair = GetDsuRoot(node.pair);
		if (rootPair.maxBud == null)
			return node;
		return rootPair.maxBud.baseNode.mate;
	}

	Node LiftNode(Node node)
	{
		NodePair rootPair = GetDsuRoot(node.pair);
		if (rootPair.maxBud == null)
			return node;
		return rootPair.maxBud.baseNode;
	}

	List<Arc> Augment(Node s)
	{
		InitAugmentation();
		s.inTree = true;
		treeNodes.Enqueue(s);

		while (treeNodes.Count > 0)
		{
			Node node = treeNodes.Dequeue();
			if (node.pair.topLevelNode == null)
				continue;
			IEnumerable<Arc> outArcs = FetchOutArcs(node);
			foreach (Arc arc in outArcs)
			{
				Node head = AntiLiftNode(arc.head);
				if (head.inTree)
					continue;
				if (head.mate.inTree)
				{
					List<Arc> path = TryTrim(arc, node, head, s);
					if (path != null)
						return path;
					break;
				}
				else
				{
					head.inTree = true;
					head.treeArc = arc;
					treeNodes.Enqueue(head);
				}
			}
		}
		return null;
	}

	void ClearTrimMarks(List<NodePair> list)
	{
		foreach (NodePair pair in list)
		{
			pair.trimMark = false;
		}
	}

	List<Arc> TryTrim(Arc connectingArc, Node tail, Node head, Node s)
	{
		Debug.Assert(tail == LiftNode(tail));
		Debug.Assert(head == AntiLiftNode(head));

		Node curTail = tail;
		Node curHead = head;
		List<NodePair> markedPairs = new List<NodePair>();
		while (curTail != null || curHead != null)
		{
			if (curTail != null)
			{
				if (curTail.pair.trimMark && curTail != s)
				{
					Trim(connectingArc, tail, head, curTail);
					ClearTrimMarks(markedPairs);
					return null;
				}
				curTail.pair.trimMark = true;
				markedPairs.Add(curTail.pair);
				curTail = curTail.treeArc == null ? null : LiftNode(curTail.treeArc.tail);
			}
			if (curHead != null)
			{
				if (curHead.pair.trimMark && curHead != s.mate)
				{
					Trim(connectingArc, tail, head, curHead.mate);
					ClearTrimMarks(markedPairs);
					return null;
				}
				curHead.pair.trimMark = true;
				markedPairs.Add(curHead.pair);
				curHead = curHead.mate.treeArc == null ? null : LiftNode(curHead.mate.treeArc.tail).mate;
			}
		}

		ClearTrimMarks(markedPairs);
		InitTimes();

		List<Node> backwardNodes = new List<Node>();
		List<Arc> backwardArcs = new List<Arc>();
		curTail = tail;
		while (curTail != null)
		{
			backwardNodes.Add(curTail);
			if (curTail.treeArc != null)
				backwardArcs.Add(curTail.treeArc);
			curTail = curTail.treeArc == null ? null : LiftNode(curTail.treeArc.tail);
		}

		List<Node> forwardNodes = new List<Node>();
		List<Arc> forwardArcs = new List<Arc>();
		forwardArcs.Add(connectingArc);
		curHead = head;
		while (curHead != null)
		{
			forwardNodes.Add(curHead);
			if (curHead.mate.treeArc != null)
				forwardArcs.Add(curHead.mate.treeArc.mate);
			curHead = curHead.mate.treeArc == null ? null : LiftNode(curHead.mate.treeArc.tail).mate;
		}

		List<Node> pathNodes = new List<Node>();
		for (int i = backwardNodes.Count - 1; i >= 0; i--)
			pathNodes.Add(backwardNodes[i]);
		for (int i = 0; i < forwardNodes.Count; i++)
			pathNodes.Add(forwardNodes[i]);

		List<Arc> pathArcs = new List<Arc>();
		for (int i = backwardArcs.Count - 1; i >= 0; i--)
			pathArcs.Add(backwardArcs[i]);
		for (int i = 0; i < forwardArcs.Count; i++)
			pathArcs.Add(forwardArcs[i]);

		Debug.Assert(pathArcs.Count == pathNodes.Count - 1);
		List<Arc> restoredPathArcs = new List<Arc>();
		for (int i = 0; i < pathNodes.Count - 1; i++)
			restoredPathArcs.AddRange(RestoreArc(pathArcs[i], pathNodes[i], pathNodes[i + 1]));

#if DEBUG
		Debug.Assert(restoredPathArcs[0].tail == s);
		Debug.Assert(restoredPathArcs[restoredPathArcs.Count - 1].head == s.mate);
		for (int i = 0; i < restoredPathArcs.Count - 1; i++)
			Debug.Assert(restoredPathArcs[i].head == restoredPathArcs[i + 1].tail);
#endif			

		Console.WriteLine("Found augmenting path: ");
		DumpPath(restoredPathArcs);
		Console.WriteLine();

		return restoredPathArcs;
	}

	void DumpPath(List<Arc> path)
	{
		for (int i = 0; i < path.Count; i++)
		{
			if (i > 0)
				Console.Write(", ");
			Arc arc = path[i];
			Console.Write("{0} -> {1}", arc.tail.index, arc.head.index);
		}
	}

	void Trim(Arc arc, Node tail, Node head, Node baseNode)
	{
		Bud bud = new Bud();
		bud.baseNode = new Node();
		bud.baseNode.mate = new Node();
		bud.baseNode.mate.mate = bud.baseNode;
		bud.baseNode.inTree = true;
		bud.baseNode.treeArc = baseNode.treeArc;

		NodePair pair = new NodePair();
		pair.first = bud.baseNode;
		pair.second = bud.baseNode.mate;
		pair.ownerBud = bud;
		bud.baseNode.pair = pair;
		bud.baseNode.mate.pair = pair;
		bud.baseArc = baseNode.treeArc;

		DsuInit(pair);
		GetDsuRoot(pair).maxBud = bud;

		List<Node> spanningNodes = new List<Node>();
		List<Arc> spanningArcs = new List<Arc>();
		Node curTail = tail;
		for (;;)
		{
			spanningNodes.Add(curTail);
			if (curTail == baseNode)
				break;
			spanningArcs.Add(curTail.treeArc);
			curTail = LiftNode(curTail.treeArc.tail);
		}

		for (int i = spanningNodes.Count - 1; i >= 0; i--)
			bud.spanningNodes.Add(spanningNodes[i]);

		for (int i = spanningArcs.Count - 1; i >= 0; i--)
			bud.spanningArcs.Add(spanningArcs[i]);

		bud.spanningArcs.Add(arc);

		Node curHead = head;
		for (;;)
		{
			curHead.pair.trimMark = true;
			if (curHead == baseNode.mate)
				break;
			bud.spanningNodes.Add(curHead);
			bud.spanningArcs.Add(curHead.mate.treeArc.mate);
			curHead = LiftNode(curHead.mate.treeArc.tail).mate;
		}

		for (int i = 1; i < bud.spanningNodes.Count; i++)
		{
			DsuUnite(bud.spanningNodes[0].pair, bud.spanningNodes[i].pair);
		}
		DsuUnite(bud.spanningNodes[0].pair, pair);

		foreach (Node spanningNode in bud.spanningNodes)
		{
			topLevelPairs.Remove(spanningNode.pair.topLevelNode);
			spanningNode.pair.topLevelNode = null;
		}

		GetDsuRoot(bud.spanningNodes[0].pair).maxBud = bud;

		pair.topLevelNode = topLevelPairs.AddLast(pair);

		treeNodes.Enqueue(bud.baseNode);
		Console.WriteLine("Trimmed {0} node(s)", bud.spanningNodes.Count * 2);
	}

	bool IsBaseNode(Node node)
	{
		return node.pair.ownerBud != null && node.pair.ownerBud.baseNode == node;
	}
	
	bool IsAntiBaseNode(Node node)
	{
		return node.pair.ownerBud != null && node.pair.ownerBud.baseNode == node.mate;
	}
	
	IEnumerable<Arc> RestoreArc(Arc arc, Node tail, Node head)
	{
		if (tail.pair.ownerBud == null && head.pair.ownerBud == null)
		{
			yield return arc;
			yield break;
		}

		if (IsBaseNode(tail))
		{
			Bud bud = tail.pair.ownerBud;
			for (int i = 0; i < bud.spanningNodes.Count; i++)
			{
				Node spanningNode = bud.spanningNodes[i];
				if (spanningNode == arc.tail ||
				    IsBaseNode(spanningNode) && Contains(spanningNode.pair.ownerBud, arc.tail))
				{
					for (int j = 0; j < i; j++)
					{
						Arc curSpanningArc = bud.spanningArcs[j];
						Node curTail = bud.spanningNodes[j];
						Node curHead = bud.spanningNodes[j + 1];
						foreach (Arc curArc in RestoreArc(curSpanningArc, curTail, curHead))
							yield return curArc;
					}
					foreach (Arc curArc in RestoreArc(arc, spanningNode, head))
						yield return curArc;
					yield break;
				}
			}

			for (int i = 0; i < bud.spanningNodes.Count; i++)
			{
				Node spanningNode = bud.spanningNodes[i].mate;
				if (spanningNode == arc.tail ||
					IsBaseNode(spanningNode) && Contains(spanningNode.pair.ownerBud, arc.tail))
				{
					for (int j = bud.spanningNodes.Count - 1; j >= i; j--)
					{
						Arc curSpanningArc = bud.spanningArcs[j].mate;
						Node curTail = (j == bud.spanningNodes.Count - 1 ? bud.spanningNodes[0] : bud.spanningNodes[j + 1].mate);
						Node curHead = bud.spanningNodes[j].mate;
						foreach (Arc curArc in RestoreArc(curSpanningArc, curTail, curHead))
							yield return curArc;
					}
					foreach (Arc curArc in RestoreArc(arc, spanningNode, head))
						yield return curArc;
					yield break;
				}
			}

			throw new NotSupportedException();
		}

		if (IsAntiBaseNode(head))
		{
			List<Arc> symArcs = new List<Arc>();
			symArcs.AddRange(RestoreArc(arc.mate, head.mate, tail.mate));
			for (int i = symArcs.Count - 1; i >= 0; i--)
				yield return symArcs[i].mate;
			yield break;
		}

		if (tail.pair.ownerBud != null && tail.pair.ownerBud.baseArc.mate == arc)
		{
			yield return arc;
			yield break;
		}

		if (head.pair.ownerBud != null && head.pair.ownerBud.baseArc == arc)
		{
			yield return arc;
			yield break;
		}

		throw new NotSupportedException();
	}

	bool Contains(Bud bud, Node node)
	{
		return node.pair.enterTime > bud.enterTime && node.pair.leaveTime < bud.leaveTime;
	}

	void InitTimes()
	{
		int time = 0;
		foreach (NodePair pair in topLevelPairs)
		{
			InitTimes(ref time, pair);
		}
	}

	void InitTimes(ref int time, NodePair pair)
	{
		pair.enterTime = time++;
		if (pair.ownerBud != null)
		{
			pair.ownerBud.enterTime = time++;
			foreach (Node spanningNode in pair.ownerBud.spanningNodes)
			{
				InitTimes(ref time, spanningNode.pair);
			}
			pair.ownerBud.leaveTime = time++;
		}
		pair.leaveTime = time++;
	}

	void ReverseArc(Arc arc)
	{
		Node head = arc.head;
		Node tail = arc.tail;

		arc.tail.outgoingArcs.Remove(arc);
		arc.mate.tail.outgoingArcs.Remove(arc.mate);

		arc.head = tail;
		arc.tail = head;

		arc.mate.head = arc.tail.mate;
		arc.mate.tail = arc.head.mate;

		arc.tail.outgoingArcs.Add(arc);
		arc.mate.tail.outgoingArcs.Add(arc.mate);
	}

	void DebugRun()
	{
		ReadRawInput();
		Augment(nodes[0]);
	}

	void Run()
	{
		ReadInput();
		answer = 0;
		for (;;)
		{
			List<Arc> path = Augment(nodes[0]);
			if (path == null)
				break;
			answer++;
			foreach (Arc arc in path)
			{
				ReverseArc(arc);
			}
		}
		WriteOutput();
	}

	void ReadInput()
	{
		using (StreamReader reader = new StreamReader("peace.in"))
		{
			string line;
			string[] tokens;

			line = reader.ReadLine();
			tokens = line.Split();

			int n = int.Parse(tokens[0]);
			int m = int.Parse(tokens[1]);
			int k = int.Parse(tokens[2]);

			for (int i = 1; i <= n + 1; i++)
			{
				AddNodePair(i);
			}

			line = reader.ReadLine();
			tokens = line.Split();
			bool[] terminals = new bool[n];
			for (int i = 0; i < k; i++)
			{
				int j = int.Parse(tokens[i]);
				terminals[j - 1] = true;
			}

			for (int i = 0; i < n; i++)
				if (terminals[i])
				{
					AddArcPair(nodes[0], nodes[i * 2 + 2]);
				}
				else
				{
					AddArcPair(nodes[i * 2 + 3], nodes[i * 2 + 2]);
				}

			for (int i = 0; i < m; i++)
			{
				line = reader.ReadLine();
				tokens = line.Split();

				int x = int.Parse(tokens[0]);
				int y = int.Parse(tokens[1]);
				AddArcPair(nodes[2 * x], nodes[2 * y + 1]);
			}
		}
	}

	void WriteOutput()
	{
		using (StreamWriter writer = new StreamWriter("peace.out"))
		{
			writer.WriteLine(answer);
		}
	}

	void ReadRawInput()
	{
		using (StreamReader reader = new StreamReader("input.txt"))
		{
			string line;
			string[] tokens;

			line = reader.ReadLine();
			tokens = line.Split();

			int n = int.Parse(tokens[0]);
			int m = int.Parse(tokens[1]);

			for (int i = 0; i < n; i++)
			{
				Node node1 = new Node();
				node1.index = i + 1;
				node1.outgoingArcs = new List<Arc>();

				Node node2 = new Node();
				node2.index = - i - 1;
				node2.outgoingArcs = new List<Arc>();

				node1.mate = node2;
				node2.mate = node1;

				NodePair pair = new NodePair();
				node1.pair = pair;
				node2.pair = pair;
				pair.first = node1;
				pair.second = node2;

				nodes.Add(node1);
				nodes.Add(node2);
			}

			for (int i = 0; i < m; i++)
			{
				line = reader.ReadLine();
				tokens = line.Split();

				int x = int.Parse(tokens[0]);
				if (x > 0)
				{
					x = 2 * (x - 1);
				}
				else
				{
					x = 2 * (-x - 1) + 1;
				}

				int y = int.Parse(tokens[1]);
				if (y > 0)
				{
					y = 2 * (y - 1);
				}
				else
				{
					y = 2 * (-y - 1) + 1;
				}

				AddArcPair(nodes[x], nodes[y]);
			}
		}
	}

	void AddArcPair(Node x, Node y)
	{
		Arc arc1 = new Arc();
		arc1.head = y;
		arc1.tail = x;

		Arc arc2 = new Arc();
		arc2.head = x.mate;
		arc2.tail = y.mate;

		arc1.mate = arc2;
		arc2.mate = arc1;

		arc1.tail.outgoingArcs.Add(arc1);
		arc2.tail.outgoingArcs.Add(arc2);

		arcs.Add(arc1);
	}

	void AddNodePair(int index)
	{
		Node node1 = new Node();
		node1.index = index;
		node1.outgoingArcs = new List<Arc>();

		Node node2 = new Node();
		node2.index = -index;
		node2.outgoingArcs = new List<Arc>();

		node1.mate = node2;
		node2.mate = node1;

		NodePair pair = new NodePair();
		node1.pair = pair;
		node2.pair = pair;
		pair.first = node1;
		pair.second = node2;

		nodes.Add(node1);
		nodes.Add(node2);
	}

	static void Main()
	{
		new Program().Run();
	}
}